use leptos::{
    IntoView,
    html::{ElementChild, button, label},
    logging::log,
    prelude::{OnAttribute, Read, Show, ShowProps, ToChildren},
    server::LocalResource,
};
use wasm_bindgen_futures::spawn_local;

use crate::{
    media::audio,
    signal::{send_auth, wait_until_communication_is_ready},
    sleep,
    webrtc::WebRTC,
};

pub fn app() -> impl IntoView {
    let audio_stream = LocalResource::new(|| audio());
    let wait_until_communication_is_ready =
        LocalResource::new(|| wait_until_communication_is_ready());

    let props = ShowProps::builder()
        .when(move || {
            audio_stream.read().is_some() && wait_until_communication_is_ready.read().is_some()
        })
        .children(ToChildren::to_children(move || {
            let audio_stream = audio_stream.read();
            let audio_stream = audio_stream.as_deref().unwrap().clone();

            let webrtc = WebRTC::new(Some(audio_stream), None, None).unwrap();
            let webrtc_state = webrtc.clone();
            spawn_local(async move {
                loop {
                    log!("{:#?}", webrtc_state.get_status());
                    sleep(1000).await;
                }
            });

            let webrtc_offer = webrtc.clone();
            let offer_button = button()
                .on(leptos::ev::click, move |_| {
                    send_auth(&String::from("Offer")).unwrap();
                    let webrtc_offer = webrtc_offer.clone();
                    spawn_local(async move {
                        if let Err(err_val) = webrtc_offer.offer().await {
                            log!("Error: WebRTC Offer | {}", err_val);
                        }
                    });
                })
                .child("Offer");

            let webrtc_answer = webrtc.clone();
            let answer_button = button()
                .on(leptos::ev::click, move |_| {
                    send_auth(&String::from("Answer")).unwrap();
                    let webrtc_answer = webrtc_answer.clone();
                    spawn_local(async move {
                        if let Err(err_val) = webrtc_answer.answer().await {
                            log!("Error: WebRTC Answer | {}", err_val);
                        }
                    });
                })
                .child("Answer");

            (offer_button, answer_button)
        }))
        .fallback(|| label().child("NOOOOOOOOOOOO"))
        .build();

    Show(props)
}
